<?php
/*
 * @copyright 2023 Passwords Handbook App
 *
 * @author Marius David Wieschollek
 * @license AGPL-3.0
 *
 * This file is part of the Passwords App
 * created by Marius David Wieschollek.
 */

namespace OCA\PasswordsHandbook\Controller;

use JetBrains\PhpStorm\Pure;
use OC\App\AppManager;
use OCA\PasswordsHandbook\AppInfo\Application;
use OCA\PasswordsHandbook\Service\FeaturesService;
use OCP\App\AppPathNotFoundException;
use OCP\AppFramework\Controller;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataDisplayResponse;
use OCP\AppFramework\Http\NotFoundResponse;
use OCP\AppFramework\Http\Response;
use OCP\IConfig;
use OCP\IRequest;
use OCP\AppFramework\Http\Attribute\NoCSRFRequired;
use OCP\AppFramework\Http\Attribute\PublicPage;
use OCP\AppFramework\Http\Attribute\NoAdminRequired;
use OCP\AppFramework\Http\Attribute\NoTwoFactorRequired;

class HandbookController extends Controller {

    /**
     * @param                 $appName
     * @param IRequest        $request
     * @param AppManager      $appManager
     * @param FeaturesService $featuresService
     */
    #[Pure] public function __construct($appName, IRequest $request, protected AppManager $appManager, protected FeaturesService $featuresService) {
        parent::__construct($appName, $request);
    }

    /**
     * @PublicPage
     * @NoCSRFRequired
     * @NoAdminRequired
     *
     * @param string $path
     *
     * @return Response
     * @throws AppPathNotFoundException
     */
    #[PublicPage]
    #[NoCSRFRequired]
    #[NoAdminRequired]
    #[NoTwoFactorRequired]
    public function get(string $path): Response {
        $data = $this->readHandbookFile($path);

        if(!$data) {
            return new NotFoundResponse();
        }
        [$content, $mime] = $data;

        return new DataDisplayResponse($content, Http::STATUS_OK, $this->getHeaders($mime));
    }

    /**
     * @param string $path
     *
     * @return object|null
     * @throws AppPathNotFoundException
     */
    protected function readHandbookFile(string $path): ?array {
        if($path === '_features/features-v1.json') {
            $data = $this->featuresService->getFeaturesJson();

            return [json_encode($data), 'application/json'];
        }

        $basePath = $this->appManager->getAppPath(Application::APP_NAME).DIRECTORY_SEPARATOR.'handbook';
        $filePath = realpath($basePath.DIRECTORY_SEPARATOR.$path);

        if(is_file($filePath) && str_starts_with($filePath, $basePath)) {
            $data = @file_get_contents($filePath);
            $mime = $this->getFileType($filePath);

            return [$data, $mime];
        }

        return null;
    }

    protected function getFileType(string $filePath): string {
        if(str_contains($filePath, '.')) {
            $fileEnding = substr($filePath, strrpos($filePath, '.') + 1);
            switch($fileEnding) {
                case 'md':
                    return 'text/plain';
                case 'json':
                    return 'application/json';
                case 'jpg':
                case 'jpeg':
                    return 'image/jpeg';
                case 'png':
                    return 'image/png';
                case 'gif':
                    return 'image/gif';
                case 'svg':
                    return 'image/svg+xml';
                case 'webm':
                    return 'video/webm';
                case 'vtt':
                    return 'text/vtt';
            }
        }

        return 'text/plain';
    }

    protected function getHeaders(mixed $mime): array {
        $requestUri = 'https://'.$this->request->getServerHost();

        return ['content-type' => $mime, 'Access-Control-Allow-Origin' => $requestUri];
    }
}