<?php

namespace OCA\PasswordsHandbook\Service;

use OC\App\AppManager;
use OCA\PasswordsHandbook\AppInfo\Application;
use OCP\App\AppPathNotFoundException;
use OCP\Files\AppData\IAppDataFactory;
use OCP\IConfig;

class FeaturesService {

    public function __construct(protected AppManager $appManager, protected IAppDataFactory $appDataFactory, protected IConfig $config) {
    }

    /**
     * @param string $client
     * @param string $name
     * @param bool   $value
     *
     * @return void
     */
    public function setFeature(string $client, string $name, bool $value): void {
        $features = $this->getFeatures();

        if(!isset($features[$client][$name])) {
            return;
        }
        $features[$client][$name] = $value;

        $this->config->setAppValue(Application::APP_NAME, 'das.features.'.$client, json_encode($features[$client]));
        $this->clearPasswordsAppCache();
    }

    /**
     * @param string $client
     * @param string $name
     *
     * @return bool
     */
    public function getFeature(string $client, string $name): bool {
        $features = $this->getFeatures();

        if(!isset($features[$client][$name])) {
            return false;
        }
        return $features[$client][$name];
    }

    /**
     * @return array
     */
    public function getFeatures(): array {
        $featuresJson = $this->getFeaturesJson();

        $features = [];
        foreach($featuresJson as $client => $versions) {
            $features[$client] = [];

            foreach($versions as $settings) {
                foreach($settings as $name => $value) {
                    $features[$client][$name] = $value;
                }
            }
        }

        return $features;
    }

    /**
     * @return array
     */
    public function getFeaturesJson(): array {
        $allOverwrites = [
            'server' => json_decode($this->config->getAppValue(Application::APP_NAME, 'das.features.server', '[]'), true),
            'webapp' => json_decode($this->config->getAppValue(Application::APP_NAME, 'das.features.webapp', '[]'), true)
        ];
        $json          = $this->readFeaturesFile();

        $appVersion     = $this->config->getAppValue('passwords', 'installed_version', date('Y.m.').'0');
        $baseAppVersion = explode('.', $appVersion);
        $baseAppVersion = $baseAppVersion[0].'.'.$baseAppVersion[1];

        $features = [];
        foreach($allOverwrites as $client => $overwrites) {
            $features[ $client ] = [];

            foreach([$baseAppVersion, $appVersion] as $version) {
                if(array_key_exists($version, $json[ $client ])) {
                    $features[ $client ][ $version ] = $json[ $client ][ $version ];

                    foreach($features[ $client ][ $version ] as $feature => $value) {
                        if(array_key_exists($feature, $overwrites)) {
                            $features[ $client ][ $version ][ $feature ] = $overwrites[ $feature ];
                        }
                    }
                }
            }
        }

        return $features;
    }

    /**
     * @return array
     */
    protected function readFeaturesFile(): array {
        try {
            $filePath = implode(DIRECTORY_SEPARATOR, [$this->appManager->getAppPath(Application::APP_NAME), 'handbook', '_features', 'features-v1.json']);
            $data     = @file_get_contents($filePath);
            if($data !== false) {
                return json_decode($data, true);
            }
        } catch(AppPathNotFoundException $e) {

        }

        return [];
    }

    /**
     * @return void
     */
    protected function clearPasswordsAppCache(): void {
        try {
            $appData = $this->appDataFactory->get('passwords');
            $appData->getFolder('defaultCache')->delete();
        } catch(\Throwable $e) {

        }
    }

}